import React, { useState, useEffect } from 'react';
import PropTypes from 'prop-types';
import { useDispatch, useSelector } from 'react-redux';
import { FormattedMessage } from 'react-intl';
import {
  Modal,
  Bullseye,
  Button,
  Radio,
  Select,
  SelectOption,
  MenuToggle,
  TextContent,
  Text,
  Tooltip,
} from '@patternfly/react-core';
import { OutlinedQuestionCircleIcon } from '@patternfly/react-icons';
import { addToast } from '../../../ToastsList/slice';
import { translate as __ } from '../../../../common/I18n';
import { STATUS } from '../../../../constants';
import {
  selectAPIStatus,
  selectAPIResponse,
} from '../../../../redux/API/APISelectors';
import {
  BULK_ASSIGN_TAXONOMY_KEY,
  bulkAssignTaxonomy,
  fetchOrganizations,
  fetchLocations,
  ORGANIZATION_KEY,
  LOCATION_KEY,
} from './actions';
import { foremanUrl } from '../../../../common/helpers';
import { APIActions } from '../../../../redux/API';
import {
  HOSTS_API_PATH,
  API_REQUEST_KEY,
} from '../../../../routes/Hosts/constants';

const BulkAssignTaxonomyModal = ({
  isOpen,
  closeModal,
  selectedCount,
  fetchBulkParams,
}) => {
  const dispatch = useDispatch();
  const [organizationId, setOrganizationId] = useState('');
  const [locationId, setLocationId] = useState('');
  const [orgSelectOpen, setOrgSelectOpen] = useState(false);
  const [locSelectOpen, setLocSelectOpen] = useState(false);
  const [orgFixRadioChecked, setOrgFixRadioChecked] = useState(true);
  const [locFixRadioChecked, setLocFixRadioChecked] = useState(true);
  const organizations = useSelector(state =>
    selectAPIResponse(state, ORGANIZATION_KEY)
  );
  const organizationStatus = useSelector(state =>
    selectAPIStatus(state, ORGANIZATION_KEY)
  );
  const locations = useSelector(state =>
    selectAPIResponse(state, LOCATION_KEY)
  );
  const locationStatus = useSelector(state =>
    selectAPIStatus(state, LOCATION_KEY)
  );
  const hostUpdateStatus = useSelector(state =>
    selectAPIStatus(state, BULK_ASSIGN_TAXONOMY_KEY)
  );
  const handleModalClose = () => {
    setOrganizationId('');
    setLocationId('');
    setOrgFixRadioChecked(true);
    setLocFixRadioChecked(true);
    closeModal();
  };

  useEffect(() => {
    dispatch(fetchOrganizations());
    dispatch(fetchLocations());
  }, [dispatch]);

  const orgNode = () => document.getElementById('organization-id');
  const orgId = () => {
    const node = orgNode();
    console.log("node", node);
    return node && node.dataset.id;
  };

  console.log("org id: ", orgId());
  const onOrgToggleClick = () => {
    setOrgSelectOpen(!orgSelectOpen);
  };
  const onOrgSelect = (_event, value) => {
    setOrganizationId(value);
    setOrgSelectOpen(false);
  };
  const toggleOrg = toggleRef => <MenuToggle ref={toggleRef} onClick={onOrgToggleClick} isExpanded={orgSelectOpen} style={{
    width: '95%'
  }}>
      {getSelectedLabel(organizationId, organizations)}
    </MenuToggle>;

  const handleOrgSelect = (event, selection) => {
    console.log("org select: ", selection);
    setOrganizationId(selection);
    setOrgSelectOpen(false);
  };

  const onLocToggleClick = () => {
    setLocSelectOpen(!locSelectOpen);
  };
  const onLocSelect = (_event, value) => {
    setLocationId(value);
    setLocSelectOpen(false);
  };
  const toggleLoc = toggleRef => <MenuToggle ref={toggleRef} onClick={onLocToggleClick} isExpanded={locSelectOpen} style={{
    width: '95%'
  }}>
      {getSelectedLabel(locationId, locations)}
    </MenuToggle>;

  const handleLocSelect = (event, selection) => {
    console.log("loc select: ", selection);
    setLocationId(selection);
    setLocSelectOpen(false);
  };

  const getSelectedLabel = (id, taxonomy) => taxonomy.results.find(t => t.id === id)?.name;

  const handleError = response => {
    let message = response.data?.error ?? response.message;
    if (message.includes('Request failed with status code 422')) {
      message = __('Update failed because of mismatch in settings');
    } else if (message.includes('Request failed with status code 400')) {
      message = __(
        'Unregister host before assigning an organization. Check server logs for more information.'
      );
    }

    dispatch(
      addToast({
        type: 'danger',
        message,
      })
    );
    handleModalClose();
  };

  const handleSuccess = response => {
    dispatch(
      addToast({
        type: 'success',
        message: response.data.message,
      })
    );
    dispatch(
      APIActions.get({
        key: API_REQUEST_KEY,
        url: foremanUrl(HOSTS_API_PATH),
      })
    );
    handleModalClose();
  };

  const handleSave = () => {
    if (locationId) {
      const requestBody = {
        included: {
          search: fetchBulkParams(),
        },
        location_id: locationId,
        optimistic_import: locFixRadioChecked ? 'yes' : 'no',
      };

      dispatch(bulkAssignTaxonomy(requestBody, handleSuccess, handleError));
    }

    if (organizationId) {
      const requestBody = {
        included: {
          search: fetchBulkParams(),
        },
        target_organization_id: organizationId,
        optimistic_import: orgFixRadioChecked ? 'yes' : 'no',
      };

      dispatch(bulkAssignTaxonomy(requestBody, handleSuccess, handleError));
    }
  };

  const handleLocationSelect = (event, selection) => {
    setLocationId(selection);
    setLocationSelectOpen(false);
  };

  const mismatchTip = taxonomy => (
    <FormattedMessage
      id="bulk-assign-org-location-mismatch-tip"
      defaultMessage={__(
        'A mismatch happens if there is a resource associated with a host, such as a domain or subnet, and at the same time not associated with the {taxonomy} you want to assign the host to. The option Fix on mismatch will add such a resource to the {taxonomy}, and is therefore the recommended choice. The option Fail on mismatch, on the other hand, will always result in an error message. For example, reassigning a host from one {taxonomy} to another will fail, even if there is no actual mismatch in settings.'
      )}
      values={{ taxonomy }}
    />
  );

  const modalActions = [
    <Button
      key="add"
      ouiaId="bulk-assign-taxonomy-modal-add-button"
      variant="primary"
      onClick={handleSave}
      isDisabled={
        hostUpdateStatus === STATUS.PENDING ||
        (organizationId === '' && locationId === '')
      }
      isLoading={hostUpdateStatus === STATUS.PENDING}
    >
      {__('Save')}
    </Button>,
    <Button
      key="cancel"
      ouiaId="bulk-assign-taxonomy-modal-cancel-button"
      variant="link"
      onClick={handleModalClose}
    >
      {__('Cancel')}
    </Button>,
  ];
  return (
    <Modal
      isOpen={isOpen}
      onClose={handleModalClose}
      onEscapePress={handleModalClose}
      title={__('Change organization/ location')}
      width="50%"
      position="top"
      actions={modalActions}
      id="bulk-assign-taxonomy-modal"
      key="bulk-assign-taxonomy-modal"
      ouiaId="bulk-assign-taxonomy-modal"
    >
      <TextContent>
        <Text ouiaId="bulk-assign-taxonomy-options">
          {__(
            'Select organization/location to add hosts to. This change may affect all your selected hosts.'
          )}
        </Text>
      </TextContent>
      {organizations && organizationStatus === STATUS.RESOLVED && (
        <>
          <h3 style={{ marginTop: '1em' }}>
            <strong>Select organization</strong>
            <Tooltip position="right" content={mismatchTip("organization")} >
              <OutlinedQuestionCircleIcon style={{ marginLeft: '4px' }} />
            </Tooltip>
          </h3>
          <Select
            id="select-organization-select"
            isOpen={orgSelectOpen}
            selected={organizationId}
            onSelect={handleOrgSelect}
            onOpenChange={isOpen => setOrgSelectOpen(isOpen)}
            toggle={toggleOrg}
            shouldFocusToggleOnSelect
          >
            {organizations.results?.map(org => (
              <SelectOption key={org.id} value={org.id}>
                {org.name}
              </SelectOption>
            ))}
          </Select>
          <Bullseye>
            <Radio
              isChecked={orgFixRadioChecked}
              name="radioOrgFixOnMismatch"
              onChange={(_e, checked) => setOrgFixRadioChecked(checked)}
              label={__('Fix on mismatch')}
              id="radio-org-fix-on-mismatch"
              ouiaId="radio-org-fix-on-mismatch"
            />
            <Radio
              isChecked={!orgFixRadioChecked}
              name="radioOrgFailOnMismatch"
              onChange={(_e, checked) => setOrgFixRadioChecked(!checked)}
              label={__('Fail on mismatch')}
              id="radio-org-fail-on-mismatch"
              ouiaId="radio-org-fail-on-mismatch"
              style={{ marginLeft: '50px' }}
            />
          </Bullseye>
        </>
      )}
      <hr />
      {locations && locationStatus === STATUS.RESOLVED && (
        <>
          <h3 style={{ marginTop: '1em' }}>
            <strong>Select location</strong>
            <Tooltip position="right" content={mismatchTip("location")} >
              <OutlinedQuestionCircleIcon style={{ marginLeft: '4px' }} />
            </Tooltip>
          </h3>
          <Select
            id="select-location-select"
            isOpen={locSelectOpen}
            selected={locationId}
            onSelect={handleLocSelect}
            onOpenChange={isOpen => setLocSelectOpen(isOpen)}
            toggle={toggleLoc}
            shouldFocusToggleOnSelect
          >
            {locations.results?.map(loc => (
              <SelectOption key={loc.id} value={loc.id}>
                {loc.name}
              </SelectOption>
            ))}
          </Select>
          <Bullseye>
            <Radio
              isChecked={locFixRadioChecked}
              name="radioLocFixOnMismatch"
              onChange={(_e, checked) => setLocFixRadioChecked(checked)}
              label={__('Fix on mismatch')}
              id="radio-loc-fix-on-mismatch"
              ouiaId="radio-loc-fix-on-mismatch"
            />
            <Radio
              isChecked={!locFixRadioChecked}
              name="radioLocFailOnMismatch"
              onChange={(_e, checked) => setLocFixRadioChecked(!checked)}
              label={__('Fail on mismatch')}
              id="radio-loc-fail-on-mismatch"
              ouiaId="radio-loc-fail-on-mismatch"
              style={{ marginLeft: '50px' }}
            />
          </Bullseye>
        </>
      )}
    </Modal>
  );
};

BulkAssignTaxonomyModal.propTypes = {
  isOpen: PropTypes.bool,
  closeModal: PropTypes.func,
  selectedCount: PropTypes.number.isRequired,
  fetchBulkParams: PropTypes.func.isRequired,
};

BulkAssignTaxonomyModal.defaultProps = {
  isOpen: false,
  closeModal: () => {},
};

export default BulkAssignTaxonomyModal;
